# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

# import os
# import tempfile
import hashlib
from .settings import SETTINGS


def update(m, x):
    if isinstance(x, (list, tuple)):
        for y in x:
            update(m, y)
        return

    if isinstance(x, dict):
        for k, v in sorted(x.items()):
            update(m, k)
            update(m, v)
        return

    m.update(str(x).encode('utf-8'))


def temp_file(*args, extension='.cache'):
    m = hashlib.sha256()
    update(m, args)
    # fd, path = tempfile.mkstemp()
    # os.close(fd)
    return "%s/climetlab-%s%s" % (SETTINGS['cache_directory'], m.hexdigest(), extension)



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#


class Driver:

    def __init__(self, *args, **kwargs):
        pass

    def bounding_box(self, north, west, south, east):
        pass

    def plot_grib(self, path, offset):
        pass

    def plot_netcdf(self, params):
        pass

    def plot_numpy(self, data, north, west, south_north_increment, west_east_increment, metadata):
        pass

    def plot_xarray(self, ds, variable, dimension_settings={}):
        pass

    def show(self, *args, **kwargs):
        pass



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

import numpy as np
import xarray as xr

from . import Dataset
from climetlab import load_source


"""
rainfall_diff_quality-code
rainfall_mean_quality-code
reflectivity_new
reflectivity_old
"""


class MeteonetRadar(Dataset):
    """
    See https://github.com/meteofrance/meteonet
    """

    def __init__(self, domain='NW', variable='rainfall', year=2016, month=8, part=3):

        URL = "https://github.com/meteofrance/meteonet/raw/master/data_samples/radar"

        url = "{url}/radar_coords_{domain}.npz".format(
            url=URL,
            domain=domain
        )

        coords = np.load(load_source("url", url).path, allow_pickle=True)

        resolution = 0.01

        lats = coords['lats'] - resolution / 2
        lons = coords['lons'] + resolution / 2

        url = "{url}/{variable}_{domain}_{year}_{month:02d}.{part}.npz".format(
            url=URL,
            domain=domain,
            variable=variable,
            year=year,
            month=month,
            part=part,
        )

        path = load_source("url", url).path
        content = np.load(path, allow_pickle=True)
        data = content['data']
        times = content['dates']
        # missing = content['miss_dates']
        # print(len(dates))
        # print(data.shape)

        self.variable = variable

        ds = xr.Dataset(
            {
                variable: (["time", "y", "x", ], data),
                "x": (["x"], range(0, data.shape[2])),
                "y": (["y"], range(0, data.shape[1])),
            },
            coords={
                "lon": (["y", "x"], lons),
                "lat": (["y", "x"], lats),
                "time": times,
            },
        )

        self.north = np.amax(lats)
        self.south = np.amin(lats)
        self.east = np.amax(lons)
        self.west = np.amin(lons)

        ds["lon"].attrs["standard_name"] = "longitude"
        ds["lat"].attrs["standard_name"] = "latitude"
        ds["time"].attrs["standard_name"] = "time"
        ds["x"].attrs["axis"] = "X"
        ds["y"].attrs["axis"] = "Y"

        self._xarray = ds

        self.path = path + ".nc"

        ds.to_netcdf(self.path)
        self.source = load_source("file", path + ".nc")

    def to_array(self):
        return self._xarray

    def plot_map(self, driver):
        driver.bounding_box(self.north, self.west,
                            self.south, self.east)

        dimensions = ["time:0"]

        driver.plot_netcdf(dict(netcdf_filename=self.path,
                                netcdf_value_variable=self.variable,
                                netcdf_dimension_setting=dimensions,
                                netcdf_dimension_setting_method='index'))

        driver.contouring(self.contouring)

    @property
    def contouring(self):
        return dict(
            contour_shade_colour_method="list",
            # contour_shade_method = "area_fill",
            contour_shade_technique='grid_shading',
            contour_shade="on",
            contour_hilo="off",
            contour="off",
            contour_highlight="off",
            contour_label="off",
            contour_shade_colour_list=['#C0C0C0',
                                       'rgba(0,0,0,0)', '#483D8B', '#0000cd', '#1E90FF',
                                       '#87ceeb', 'olive', '#3cb371', 'cyan', '#00FF00', 'yellow',
                                       'khaki', 'burlywood', 'orange', 'brown', 'pink', 'red', 'plum'],
            contour_level_list=[float(x) for x in [-1, 0, 2, 4, 6, 8, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 75]],
            #                contour_shade_min_level=0.5
        )


dataset = MeteonetRadar



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

class NumpyArrayPlotter:

    def __init__(self, data, field, **kwargs):
        self.data = data
        self.field = field

    def plot_map(self, driver):
        grid = self.field.grid_definition()

        driver.bounding_box(north=grid['north'],
                            south=grid['south'],
                            west=grid['west'],
                            east=grid['east'])

        driver.plot_numpy(self.data,
                          north=grid['north'],
                          west=grid['west'],
                          south_north_increment=grid['south_north_increment'],
                          west_east_increment=grid['west_east_increment'],
                          metadata=self.field.metadata(),
                          )


helper = NumpyArrayPlotter



#!/usr/bin/env python3

# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#


import io
import os.path

import setuptools


def read(fname):
    file_path = os.path.join(os.path.dirname(__file__), fname)
    return io.open(file_path, encoding='utf-8').read()


version = None
for line in read('climetlab/__init__.py').split('\n'):
    if line.startswith("__version__"):
        version = line.split('=')[-1].strip()[1:-1]


assert version

setuptools.setup(
    name='climetlab',
    version=version,
    author='ECMWF',
    author_email='software.support@ecmwf.int',
    license='Apache 2.0',
    url='https://github.com/ecmwf/climetlab',
    description="Handling of climate/meteorological data",
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=[
        'pandas==1.0.5',  # Magics xarray does not work with pandas 1.1.0
        'numpy',
        'xarray',
        'netcdf4',
        'cfgrib',
        'cdsapi',
        'ecmwf-api-client',
        'pandas',
        'tqdm',
        'requests',
        'eccodes',
        'magics',
        'ecmwflibs',
        'pdbufr',
        'pyodc',
        'dask',
        'toolz',
    ],
    zip_safe=True,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Operating System :: OS Independent',
    ],
)



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

class PandasPlotter:

    def __init__(self, frame):
        self.frame = frame
        self.lat = "lat"
        self.lon = "lon"

    def plot_map(self, driver):
        north, east = self.frame[[self.lat, self.lon]].max()
        south, west = self.frame[[self.lat, self.lon]].min()
        driver.bounding_box(north=north,
                            south=south,
                            west=west,
                            east=east)

        path = "tmp.geo"
        # driver.plot_grib(self.path, self.handle.get('offset'))
        with open(path, "w") as f:
            print("#GEO", file=f)
            print("#lat long value", file=f)
            print("#DATA", file=f)

            for index, row in self.frame[[self.lat, self.lon]].iterrows():
                print(row[0], row[1], 42.0, file=f)

        driver.plot_geopoints(path)


helper = PandasPlotter



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

import os


def grib_reader(path):
    from .grib import GRIBReader
    return GRIBReader(path)


def bufr_reader(path):
    from .bufr import BUFRReader
    return BUFRReader(path)


def netcdf_reader(path):
    from .netcdf import NetCDFReader
    return NetCDFReader(path)


def odb_reader(path):
    from .odb import ODBReader
    return ODBReader(path)


def csv_reader(path):
    from .csv import CSVReader
    return CSVReader(path)


READERS = {
    b'GRIB': grib_reader,
    b'BUFR': bufr_reader,
    b'\x89HDF': netcdf_reader,
    b'CDF\x01': netcdf_reader,
    b'CDF\x02': netcdf_reader,
    b'\xff\xffOD': odb_reader,

    '.csv': csv_reader,
}


def reader(path):

    _, extension = os.path.splitext(path)
    if extension in READERS:
        return READERS[extension](path)

    with open(path, 'rb') as f:
        header = f.read(4)

    if header in READERS:
        return READERS[header](path)

    raise ValueError('Unsupported file {} (header={}, extension={})'.format(path, header, extension))



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

import pandas as pd

from . import Dataset
from climetlab import load_source


class MeteonetGroundStations(Dataset):
    """
    See https://github.com/meteofrance/meteonet
    """

    def __init__(self, domain='NW', date='20160101'):

        URL = "https://github.com/meteofrance/meteonet/raw/master/data_samples/ground_stations"
        url = "{url}/{domain}_{date}.csv".format(url=URL, domain=domain, date=date)
        self._pandas = pd.read_csv(load_source("url", url).path, parse_dates=[4], infer_datetime_format=True)

    def to_pandas(self):
        return self._pandas


dataset = MeteonetGroundStations
